@@ -35,6 +35,14 @@ common_args = dict(
 )
 
 
+def get_statements(conn):
+    """
+    Gets all statements as a list of lists of Value objects,
+    sorted lexicographically.
+    """
+    return sorted([list(s) for s in conn.getStatements()])
+
+
 def test_ag_connect_open(repo_name):
     with ag_connect(repo_name, create=False, **common_args) as conn:
         assert conn.size() == 0
@@ -460,3 +468,64 @@ def test_export_to_true(capfd, conn, s, p, o):
     conn.getStatements(output=True)
     out, err = capfd.readouterr()
     assert out.strip() == "<ex://s> <ex://p> <ex://o> ."
+
+
+def test_triple_len(s, p, o):
+    assert len(Statement(s, p, o)) == 3
+
+
+def test_quad_len(s, p, o, g):
+    assert len(Statement(s, p, o, g)) == 4
+
+
+def test_stmt_str(ex):
+    # This used to raise an error, see bug25079
+    assert 'ex' in str(Statement(ex.s, ex.p, ex.o))
+
+
+def test_statement_from_strings_subject(ex):
+    stmt = Statement('<ex://s>', '<ex://p>', '<ex://o>')
+    assert stmt.getSubject() == ex.s
+
+
+def test_statement_from_strings_predicate(ex):
+    stmt = Statement('<ex://s>', '<ex://p>', '<ex://o>')
+    assert stmt.getPredicate() == ex.p
+
+
+def test_statement_from_strings_object(ex):
+    stmt = Statement('<ex://s>', '<ex://p>', '<ex://o>')
+    assert stmt.getObject() == ex.o
+
+
+def test_triple_ne_quad(ex):
+    s1 = Statement(ex.s, ex.p, ex.o)
+    s2 = Statement(ex.s, ex.p, ex.o, ex.g)
+    assert s1 != s2
+
+
+def test_add_triples_statements(conn, ex):
+    s = Statement(ex.s, ex.p, ex.o)
+    conn.addTriples([s])
+    assert conn.getStatements().asList() == [s]
+
+
+def test_conn_namespace(conn):
+    ex = conn.namespace('http://franz.com/example/')
+    assert ex.foo == conn.createURI('http://franz.com/example/foo')
+
+
+def test_add_with_single_context(conn, s, p, o, g):
+    conn.addTriples([(s, p, o)], context=g)
+    assert get_statements(conn) == [[s, p, o, g]]
+
+
+def test_add_with_two_contexts(conn, ex):
+    conn.addTriples([(ex.s, ex.p, ex.o)], context=[ex.g1, ex.g2])
+    assert get_statements(conn) == [[ex.s, ex.p, ex.o, ex.g1],
+                                    [ex.s, ex.p, ex.o, ex.g2]]
+
+
+def test_add_quad_overrides_contexts(conn, ex):
+    conn.addTriples([(ex.s, ex.p, ex.o, ex.g)], context=[ex.g1, ex.g2])
+    assert get_statements(conn) == [[ex.s, ex.p, ex.o, ex.g]]
